%define anolis_release 6

%global toolchain clang
 
%undefine _include_frame_pointers

%bcond_with compat_build
%bcond_without check

%global maj_ver 17
%global min_ver 0
%global patch_ver 6
%global clang_version %{maj_ver}.%{min_ver}.%{patch_ver}

%global clang_srcdir clang-%{clang_version}.src
%global cmake_srcdir cmake-%{clang_version}.src
%global clang_tools_srcdir clang-tools-extra-%{clang_version}.src

Name:		clang
Version:	%{clang_version}
Release:	%{anolis_release}%{?dist}
Summary:	A C language family front-end for LLVM

License:	Apache-2.0 WITH LLVM-exception OR NCSA
URL:		http://llvm.org
Source0:	https://github.com/llvm/llvm-project/releases/download/llvmorg-%{clang_version}/%{clang_srcdir}.tar.xz
Source1:        https://github.com/llvm/llvm-project/releases/download/llvmorg-%{clang_version}/%{clang_tools_srcdir}.tar.xz
Source4:        https://github.com/llvm/llvm-project/releases/download/llvmorg-%{clang_version}/%{cmake_srcdir}.tar.xz
Source7:	macros.%{name}

# Patches for clang
Patch1:     0001-PATCH-clang-Make-funwind-tables-the-default-on-all-a.patch
Patch2:     0003-PATCH-clang-Don-t-install-static-libraries.patch
Patch3:     0001-Driver-Add-a-gcc-equivalent-triple-to-the-list-of-tr.patch
# Drop the following patch after debugedit adds support to DWARF-5:
# https://sourceware.org/bugzilla/show_bug.cgi?id=28728
Patch4:     0001-Produce-DWARF4-by-default.patch
# Workaround a bug in ORC on ppc64le.
# More info is available here: https://reviews.llvm.org/D159115#4641826
Patch5:     0001-Workaround-a-bug-in-ORC-on-ppc64le.patch
# Patches for https://issues.redhat.com/browse/RHEL-1650
# Remove in clang 18.
Patch6:     cfg.patch
Patch7:     tsa.patch


# RHEL specific patches
# Avoid unwanted dependency on python-recommonmark
Patch101:  0009-disable-recommonmark.patch

# Add anolis support in driver
Patch11:    1001-add-anolis-os-support-in-driver.patch
Patch12:    0001-Clang-LoongArch-Add-the-triple-for-anolis-os.patch

# Patches for LoongArch
Patch14:    0001-Clang-LoongArch-Use-the-ClangBuiltin-class-to-automa.patch
Patch15:    0002-Clang-LoongArch-Support-compiler-options-mlsx-mlasx-.patch
Patch16:    0003-Clang-LoongArch-Add-ABI-implementation-of-passing-ve.patch
Patch17:    0004-Clang-LoongArch-Support-the-builtin-functions-for-LS.patch
Patch18:    0005-Clang-LoongArch-Support-the-builtin-functions-for-LA.patch
Patch19:    0006-LoongArch-CodeGen-Add-LSX-builtin-testcases.patch
Patch20:    0007-LoongArch-CodeGen-Add-LASX-builtin-testcases.patch
Patch21:    0008-Clang-LoongArch-Do-not-pass-vector-arguments-via-vec.patch
Patch22:    0009-Driver-Support-fsanitize-cfi-icall-on-loongarch64-67.patch
Patch23:    0010-Clang-LoongArch-Generate-_mcount-instead-of-mcount-6.patch
Patch24:    0011-LoongArch-test-Add-some-ABI-regression-tests-for-emp.patch
Patch25:    0012-LoongArch-Fix-ABI-mismatch-with-gcc-g-about-empty-st.patch
Patch26:    0013-LoongArch-Pre-commit-test-for-issue-70890.patch
Patch27:    0014-LoongArch-Fix-ABI-mismatch-with-g-when-handling-empt.patch
Patch28:    0015-Driver-Default-LoongArch-to-fno-direct-access-extern.patch
Patch29:    0016-Clang-LoongArch-Precommit-test-for-fix-wrong-return-.patch
Patch30:    0017-Clang-LoongArch-Fix-wrong-return-value-type-of-__ioc.patch
Patch31:    0018-Driver-Support-mcmodel-for-LoongArch-72514.patch
Patch32:    0019-LoongArch-clang-Add-support-for-option-msimd-and-mac.patch
Patch33:    0020-LoongArch-clang-Modify-loongarch-msimd.c-to-avoid-gr.patch
Patch34:    0021-LoongArch-CodeGen-Implement-128-bit-and-256-bit-vect.patch
Patch35:    0022-LoongArch-Enable-128-bits-vector-by-default-100056.patch
#Patch36:    0023-LoongArch-Add-definitions-and-feature-frecipe-for-FP.patch
#Patch37:    0024-LoongArch-Support-march-la64v1.0-and-march-la64v1.1-.patch
#Patch38:    0025-LoongArch-Support-la664-100068.patch
#Patch39:    0026-LoongArch-Fix-test-issue-of-init-loongarch.c.patch

# Patches for clang-tools-extra
# See https://reviews.llvm.org/D120301
Patch201:   0001-clang-tools-extra-Make-test-dependency-on-LLVMHello-.patch

BuildRequires:	clang
BuildRequires:	cmake
BuildRequires:	ninja-build
BuildRequires:	llvm-devel = %{version}
BuildRequires:	llvm-test = %{version}
# llvm-static is required, because clang-tablegen needs libLLVMTableGen, which
# is not included in libLLVM.so.
BuildRequires:	llvm-static = %{version}
BuildRequires:	llvm-googletest = %{version}

BuildRequires:	libxml2-devel
BuildRequires:  libffi-devel
BuildRequires:	perl-generators
BuildRequires:	ncurses-devel
BuildRequires:	emacs

# The testsuite uses /usr/bin/lit which is part of the python3-lit package.
BuildRequires:	python3-lit >= %{maj_ver}

BuildRequires:	python3-sphinx
BuildRequires:  python3-recommonmark
BuildRequires:	libatomic

# We need python3-devel for %%py3_shebang_fix
BuildRequires:	python3-devel

# For reproducible pyc file generation
BuildRequires: /usr/bin/marshalparser
%global py_reproducible_pyc_path %{buildroot}%{python3_sitelib}

# scan-build uses these perl modules so they need to be installed in order
# to run the tests.
BuildRequires: perl(Digest::MD5)
BuildRequires: perl(File::Copy)
BuildRequires: perl(File::Find)
BuildRequires: perl(File::Path)
BuildRequires: perl(File::Temp)
BuildRequires: perl(FindBin)
BuildRequires: perl(Hash::Util)
BuildRequires: perl(lib)
BuildRequires: perl(Term::ANSIColor)
BuildRequires: perl(Text::ParseWords)
BuildRequires: perl(Sys::Hostname)

Requires:	%{name}-libs = %{version}-%{release}

# clang requires gcc, clang++ requires libstdc++-devel
Requires:	libstdc++-devel
Requires:	gcc-c++

Provides:	clang(major) = %{maj_ver}

Conflicts:	compiler-rt < 11.0.0

%description
clang: noun
    1. A loud, resonant, metallic sound.
    2. The strident call of a crane or goose.
    3. C-language family front-end toolkit.

The goal of the Clang project is to create a new C, C++, Objective C
and Objective C++ front-end for the LLVM compiler. Its tools are built
as libraries and designed to be loosely-coupled and extensible.

Install compiler-rt if you want the Blocks C language extension or to
enable sanitization and profiling options when building, and
libomp-devel to enable -fopenmp.

%package libs
Summary: Runtime library for clang
Requires: %{name}-resource-filesystem = %{version}
Recommends: compiler-rt = %{version}
Recommends: libatomic
# libomp-devel is required, so clang can find the omp.h header when compiling
# with -fopenmp.
Recommends: libomp-devel = %{version}
Recommends: libomp = %{version}

%description libs
Runtime library for clang.

%package devel
Summary: Development header files for clang
Requires: %{name} = %{version}-%{release}
# The clang CMake files reference tools from clang-tools-extra.
Requires: %{name}-tools-extra = %{version}-%{release}
Requires: %{name}-libs = %{version}-%{release}
Requires: cmake(llvm) = %{version}

%description devel
Development header files for clang.

%package resource-filesystem
Summary: Filesystem package that owns the clang resource directory
Provides: %{name}-resource-filesystem(major) = %{maj_ver}

%description resource-filesystem
This package owns the clang resouce directory: $libdir/clang/$version/

%package analyzer
Summary:	A source code analysis framework
License:	NCSA and MIT
BuildArch:	noarch
Requires:	%{name} = %{version}-%{release}

%description analyzer
The Clang Static Analyzer consists of both a source code analysis
framework and a standalone tool that finds bugs in C and Objective-C
programs. The standalone tool is invoked from the command-line, and is
intended to run in tandem with a build of a project or code base.

%package tools-extra
Summary:	Extra tools for clang
Requires:	%{name}-libs = %{version}-%{release}
Requires:	emacs-filesystem

%description tools-extra
A set of extra tools built using Clang's tooling API.

%package tools-extra-devel
Summary: Development header files for clang tools
Requires: %{name}-tools-extra = %{version}-%{release}
 
%description tools-extra-devel
Development header files for clang tools.

# Put git-clang-format in its own package, because it Requires git
# and we don't want to force users to install all those dependenices if they
# just want clang.
%package -n git-clang-format
Summary:	Integration of clang-format for git
Requires:	%{name}-tools-extra = %{version}-%{release}
Requires:	git
Requires:	python3

%description -n git-clang-format
clang-format integration for git.

%package -n python3-clang
Summary:        Python3 bindings for clang
Requires:       %{name}-libs = %{version}-%{release}
Requires:       python3

%description -n python3-clang
%{summary}.

%prep
%setup -T -q -b 4 -n %{cmake_srcdir}
# TODO: It would be more elegant to set -DLLVM_COMMON_CMAKE_UTILS=%{_builddir}/%{cmake_srcdir},
# but this is not a CACHED variable, so we can't actually set it externally :(
cd ..
mv %{cmake_srcdir} cmake
%if %{with compat_build}
%autosetup -n %{clang_srcdir} -p2
%else
 
%setup -T -q -b 1 -n %{clang_tools_srcdir}
%autopatch -m200 -p2
 
 
# failing test case
rm test/clang-tidy/checkers/altera/struct-pack-align.cpp
 
%py3_shebang_fix \
        clang-tidy/tool/ \
        clang-include-fixer/find-all-symbols/tool/run-find-all-symbols.py
 
%setup -q -n %{clang_srcdir}
%autopatch -M200 -p2
 
# failing test case
rm test/CodeGen/profile-filter.c
 
%py3_shebang_fix \
        tools/clang-format/ \
        tools/clang-format/git-clang-format \
        utils/hmaptool/hmaptool \
        tools/scan-view/bin/scan-view \
        tools/scan-view/share/Reporter.py \
        tools/scan-view/share/startfile.py \
        tools/scan-build-py/bin/* \
        tools/scan-build-py/libexec/*
%endif

%build
# Use ThinLTO to limit build time.
%define _lto_cflags -flto=thin
# And disable LTO on AArch64 entirely.
%ifarch aarch64
%define _lto_cflags %{nil}
%endif

sed -i 's/\@FEDORA_LLVM_LIB_SUFFIX\@/64/g' test/lit.cfg.py

%ifarch aarch64
# Decrease debuginfo verbosity to reduce memory consumption during final library linking
%global optflags %(echo %{optflags} | sed 's/-g /-g1 /')
 
# Disable dwz on aarch64, because it takes a huge amount of time to decide not to optimize things.
%define _find_debuginfo_dwz_opts %{nil}
%endif

# We set CLANG_DEFAULT_PIE_ON_LINUX=OFF and PPC_LINUX_DEFAULT_IEEELONGDOUBLE=ON to match the
# defaults used by our GCC.
%cmake  -G Ninja \
	-DCLANG_DEFAULT_PIE_ON_LINUX=OFF \
        -DPPC_LINUX_DEFAULT_IEEELONGDOUBLE=ON \
	-DLLVM_PARALLEL_LINK_JOBS=1 \
	-DLLVM_LINK_LLVM_DYLIB:BOOL=ON \
	-DCMAKE_BUILD_TYPE=RelWithDebInfo \
	-DPYTHON_EXECUTABLE=%{__python3} \
	-DCMAKE_SKIP_RPATH:BOOL=ON \
	-DCLANG_INCLUDE_TESTS:BOOL=ON \
	-DLLVM_BUILD_UTILS:BOOL=ON \
	-DLLVM_EXTERNAL_CLANG_TOOLS_EXTRA_SOURCE_DIR=../%{clang_tools_srcdir} \
	-DLLVM_EXTERNAL_LIT=%{_bindir}/lit \
        -DLLVM_LIT_ARGS="-vv" \
	-DLLVM_MAIN_SRC_DIR=%{_datadir}/llvm/src \
	-DLLVM_LIBDIR_SUFFIX=64 \
        \
	-DLLVM_TABLEGEN_EXE:FILEPATH=%{_bindir}/llvm-tblgen \
	-DCLANG_ENABLE_ARCMT:BOOL=ON \
	-DCLANG_ENABLE_STATIC_ANALYZER:BOOL=ON \
	-DCLANG_INCLUDE_DOCS:BOOL=ON \
	-DCLANG_PLUGIN_SUPPORT:BOOL=ON \
	-DENABLE_LINKER_BUILD_ID:BOOL=ON \
	-DLLVM_ENABLE_EH=ON \
	-DLLVM_ENABLE_RTTI=ON \
	-DLLVM_BUILD_DOCS=ON \
	-DLLVM_ENABLE_SPHINX=ON \
	-DCLANG_LINK_CLANG_DYLIB=ON \
	-DSPHINX_WARNINGS_AS_ERRORS=OFF \
	\
	-DCLANG_BUILD_EXAMPLES:BOOL=OFF \
	-DBUILD_SHARED_LIBS=OFF \
	-DCLANG_REPOSITORY_STRING="anolis os %{version}-%{release}" \
	-DCLANG_DEFAULT_UNWINDLIB=libgcc

%cmake_build

%install

%cmake_install

# File in the macros file for other packages to use.  We are not doing this
# in the compat package, because the version macros would # conflict with
# eachother if both clang and the clang compat package were installed together.
install -p -m0644 -D %{SOURCE7} %{buildroot}%{_rpmmacrodir}/macros.%{name}
sed -i -e "s|@@CLANG_MAJOR_VERSION@@|%{maj_ver}|" \
       -e "s|@@CLANG_MINOR_VERSION@@|%{min_ver}|" \
       -e "s|@@CLANG_PATCH_VERSION@@|%{patch_ver}|" \
       %{buildroot}%{_rpmmacrodir}/macros.%{name}

# install clang python bindings
mkdir -p %{buildroot}%{python3_sitelib}/clang/
install -p -m644 bindings/python/clang/* %{buildroot}%{python3_sitelib}/clang/
%py_byte_compile %{__python3} %{buildroot}%{python3_sitelib}/clang

# install scanbuild-py to python sitelib.
mv %{buildroot}%{_prefix}/%{_lib}/{libear,libscanbuild} %{buildroot}%{python3_sitelib}
%py_byte_compile %{__python3} %{buildroot}%{python3_sitelib}/{libear,libscanbuild}

# Fix permissions of scan-view scripts
chmod a+x %{buildroot}%{_datadir}/scan-view/{Reporter.py,startfile.py}

# Move emacs integration files to the correct directory
mkdir -p %{buildroot}%{_emacs_sitestartdir}
for f in clang-format.el clang-rename.el clang-include-fixer.el; do
mv %{buildroot}{%{_datadir}/clang,%{_emacs_sitestartdir}}/$f
done

# remove editor integrations (bbedit, sublime, emacs, vim)
rm -vf %{buildroot}%{_datadir}/clang/clang-format-bbedit.applescript
rm -vf %{buildroot}%{_datadir}/clang/clang-format-sublime.py*

# TODO: Package html docs
rm -Rvf %{buildroot}%{_docdir}/Clang/clang/html
rm -Rvf %{buildroot}%{_datadir}/clang/clang-doc-default-stylesheet.css
rm -Rvf %{buildroot}%{_datadir}/clang/index.js

rm -vf %{buildroot}%{_datadir}/clang/bash-autocomplete.sh

# Create Manpage symlinks
ln -s clang.1 %{buildroot}%{_mandir}/man1/clang++.1
ln -s clang.1 %{buildroot}%{_mandir}/man1/clang-%{maj_ver}.1
ln -s clang.1 %{buildroot}%{_mandir}/man1/clang++-%{maj_ver}.1

# Add clang++-{version} symlink
ln -s clang++ %{buildroot}%{_bindir}/clang++-%{maj_ver}


# Fix permission
chmod u-x %{buildroot}%{_mandir}/man1/scan-build.1*

# Create sub-directories in the clang resource directory that will be
# populated by other packages
mkdir -p %{buildroot}%{_libdir}/clang/%{maj_ver}/{include,lib,share}/

# Add a symlink in /usr/bin to clang-format-diff
ln -s %{_datadir}/clang/clang-format-diff.py %{buildroot}%{_bindir}/clang-format-diff

%check
%if %{with check}
# Build test dependencies separately, to prevent invocations of host clang from being affected
# by LD_LIBRARY_PATH below.
%cmake_build --target clang-test-depends \
    ExtraToolsUnitTests ClangdUnitTests ClangIncludeCleanerUnitTests ClangPseudoUnitTests
# requires lit.py from LLVM utilities
# FIXME: Fix failing ARM tests
LD_LIBRARY_PATH=%{buildroot}/%{_libdir} %{__ninja} check-all -C %{__cmake_builddir} || true
%endif

%files
%license LICENSE.TXT
%{_bindir}/clang
%{_bindir}/clang++
%{_bindir}/clang-%{maj_ver}
%{_bindir}/clang++-%{maj_ver}
%{_bindir}/clang-cl
%{_bindir}/clang-cpp
%{_mandir}/man1/clang.1*
%{_mandir}/man1/clang++.1*
%{_mandir}/man1/clang-%{maj_ver}.1*
%{_mandir}/man1/clang++-%{maj_ver}.1*

%files libs
%{_libdir}/clang/%{maj_ver}/include/*
%{_libdir}/*.so.*

%files devel
%{_libdir}/*.so
%{_includedir}/clang/
%{_includedir}/clang-c/
%{_libdir}/cmake/*
%{_bindir}/clang-tblgen
%dir %{_datadir}/clang/
%{_rpmmacrodir}/macros.%{name}

%files resource-filesystem
%dir %{_libdir}/clang/
%dir %{_libdir}/clang/%{maj_ver}/
%dir %{_libdir}/clang/%{maj_ver}/include/
%dir %{_libdir}/clang/%{maj_ver}/lib/
%dir %{_libdir}/clang/%{maj_ver}/share/

%files analyzer
%{_bindir}/scan-view
%{_bindir}/scan-build
%{_bindir}/analyze-build
%{_bindir}/intercept-build
%{_bindir}/scan-build-py
%{_libexecdir}/ccc-analyzer
%{_libexecdir}/c++-analyzer
%{_libexecdir}/analyze-c++
%{_libexecdir}/analyze-cc
%{_libexecdir}/intercept-c++
%{_libexecdir}/intercept-cc
%{_datadir}/scan-view/
%{_datadir}/scan-build/
%{_mandir}/man1/scan-build.1.*
%{python3_sitelib}/libear
%{python3_sitelib}/libscanbuild


%files tools-extra
%{_bindir}/amdgpu-arch
%{_bindir}/clang-apply-replacements
%{_bindir}/clang-change-namespace
%{_bindir}/clang-check
%{_bindir}/clang-doc
%{_bindir}/clang-extdef-mapping
%{_bindir}/clang-format
%{_bindir}/clang-include-cleaner
%{_bindir}/clang-include-fixer
%{_bindir}/clang-move
%{_bindir}/clang-offload-bundler
%{_bindir}/clang-offload-packager
%{_bindir}/clang-linker-wrapper
%{_bindir}/clang-pseudo
%{_bindir}/clang-query
%{_bindir}/clang-refactor
%{_bindir}/clang-rename
%{_bindir}/clang-reorder-fields
%{_bindir}/clang-repl
%{_bindir}/clang-scan-deps
%{_bindir}/clang-tidy
%{_bindir}/clangd
%{_bindir}/diagtool
%{_bindir}/hmaptool
%{_bindir}/nvptx-arch
%{_bindir}/pp-trace
%{_bindir}/c-index-test
%{_bindir}/find-all-symbols
%{_bindir}/modularize
%{_bindir}/clang-format-diff
%{_mandir}/man1/diagtool.1*
%{_emacs_sitestartdir}/clang-format.el
%{_emacs_sitestartdir}/clang-rename.el
%{_emacs_sitestartdir}/clang-include-fixer.el
%{_datadir}/clang/clang-format.py*
%{_datadir}/clang/clang-format-diff.py*
%{_datadir}/clang/clang-include-fixer.py*
%{_datadir}/clang/clang-tidy-diff.py*
%{_bindir}/run-clang-tidy
%{_datadir}/clang/run-find-all-symbols.py*
%{_datadir}/clang/clang-rename.py*

%files tools-extra-devel
%{_includedir}/clang-tidy/

%files -n git-clang-format
%{_bindir}/git-clang-format

%files -n python3-clang
%{python3_sitelib}/clang/

%changelog
* Thu Oct 31 2024 Chen Li <chenli@loongson.cn> - 17.0.6-6
- LoongArch Backport: Add support for option `-msimd=` and macro
  `__loongarch_simd_width`
- LoongArch Backport: Enable 128-bits vector by default
- LoongArch Backport:  Add definitions and feature 'frecipe' for FP
  approximation builtins
- LoongArch Backport: Support -march=la64v1.0 and -march=la64v1.1
- LoongArch Backport: Support la664

* Wed Oct 23 2024 Chen Li <chenli@loongson.cn> - 17.0.6-5
- LoongArch Backport: Modify -mcmodel options to be consistent with GCC

* Tue Apr 9 2024 Chen Li <chenli@loongson.cn> - 17.0.6-4
- LoongArch Backport: Add the support for vector(LSX/LASX)
- LoongArch Backport: Support -fsanitize=cfi-icall
- LoongArch Backport: Bugfix

* Mon Apr 1 2024 Chen Li <chenli@loongson.cn> - 17.0.6-3
- LoongArch: Add the triple for anolis os
- LoongArch: Fix Segmentation fault when pie is disabled

* Mon Mar 25 2024 Bo Ren <rb01097748@alibaba-inc.com> - 17.0.6-2
- Rebuild with python3.11

* Tue Mar 12 2024 Xiaoping Liu <lxp01404969@alibaba-inc.com> - 17.0.6-1
- update to 17.0.6

* Fri Aug 11 2023 Chen Li <chenli@loongson.cn> - 16.0.6-2
- Backport LoongArch patches from llvmorg-17.0.0-rc2
- Add the triple of loongarch64 for anolis os
- LoongArch: fix symbol lookup error when compiling on aarch and x86

* Tue Aug 8 2023 Funda Wang <fundawang@yeah.net> - 16.0.6-1
- New version 16.0.6

* Fri May 19 2023 Funda Wang <fundawang@yeah.net> - 16.0.4-2
- More strict llvm version requirement on cmake-devel

* Thu May 18 2023 Funda Wang <fundawang@yeah.net> - 16.0.4-1
- New version 16.0.4

* Tue Apr 18 2023 Funda Wang <fundawang@yeah.net> - 16.0.1-2
- Rebuild with check

* Wed Apr 5 2023 Funda Wang <fundawang@yeah.net> - 16.0.1-1
- New version 16.0.1

* Tue Mar 28 2023 mgb01105731 <mgb01105731@alibaba-inc.com> - 15.0.7-4
- Del Specific Fields

* Sun Feb 12 2023 Funda Wang <fundawang@yeah.net> - 15.0.7-3
- Rebuild with clang 15

* Wed Feb 1 2023 Funda Wang <fundawang@yeah.net> - 15.0.7-2
- Rebuild with clang

* Mon Jan 30 2023 Funda Wang <fundawang@yeah.net> - 15.0.7-1
- New version 15.0.7

* Tue Jun 21 2022 happy_orange <songnannan@linux.alibaba.com> - 13.0.1-2
- add anolis driver support

* Mon Apr 25 2022 happy_orange <songnannan@linux.alibaba.com> - 13.0.1-1
- package init form upstream
